@@ -484,7 +484,7 @@ class ScreenerData:
         for i in range(10):
             url = self.url + '?page=' + str(i+1)
             df = self._getData(url)
-            if int(df['S.No.'].values[0]) != 25*(i) + 1:
+            if not df.empty and int(df['S.No.'].values[0]) != 25*(i) + 1:
                 break
             else:
                 data = pd.concat([data, df])
@@ -498,33 +498,36 @@ class ScreenerData:
         result = [x.find("tbody") for x in soup.find_all("table")]
         
         data = []
-        # Headers
-        headers = result[0].find("tr").find_all("th")
-        headers = [y.find('a').children for y in headers]
-        headers = [next(z).strip() for z in headers]
+        try:
+            # Headers
+            headers = result[0].find("tr").find_all("th")
+            headers = [y.find('a').children for y in headers]
+            headers = [next(z).strip() for z in headers]
 
-        # Data
-        comps = result[0].find_all("tr")[1:]
-        for comp in comps:
-            details = comp.find_all("td")
-            i = 0
-            temp = {}
-            for detail in details:
-                try:
-                    if detail.find('a') is not None:
-                        temp['Symbol'] = detail.find('a').attrs['href'].split('/')[2]
-                        value = next(detail.find('a').children).strip()
-                    else:
-                        value = next(detail.children).strip()
+            # Data
+            comps = result[0].find_all("tr")[1:]
+            for comp in comps:
+                details = comp.find_all("td")
+                i = 0
+                temp = {}
+                for detail in details:
                     try:
-                        value = float(value)
+                        if detail.find('a') is not None:
+                            temp['Symbol'] = detail.find('a').attrs['href'].split('/')[2]
+                            value = next(detail.find('a').children).strip()
+                        else:
+                            value = next(detail.children).strip()
+                        try:
+                            value = float(value)
+                        except:
+                            pass
                     except:
-                        pass
-                except:
-                    value = ''
-                temp[headers[i]] = value
-                i += 1
-            data.append(temp)
+                        value = ''
+                    temp[headers[i]] = value
+                    i += 1
+                data.append(temp)
+        except:
+            pass
 
         data = pd.DataFrame(data)
         return data
@@ -532,84 +535,112 @@ class ScreenerData:
 class MagicFormulaStrategy:
     
     def screener(self, formatMessage = True):
-        url = 'https://www.screener.in/screens/450341/Magic-Formula/'
-        obj = ScreenerData(url)
-        data = obj.get()
-        data = data.sort_values(by = ['Earnings Yield', 'ROCE'], ascending = False)
-        data.reset_index( inplace = True, drop = True)
+        try:
+            url = 'https://www.screener.in/screens/450341/Magic-Formula/'
+            obj = ScreenerData(url)
+            data = obj.get()
+            data = data.sort_values(by = ['Earnings Yield', 'ROCE'], ascending = False)
+            data.reset_index( inplace = True, drop = True)
 
-        # Strategy
-        amount_per_stock = 10000
-        data['Quantity'] = round(amount_per_stock/data['CMP'],0)
-        
-        if formatMessage:
-            message = '<b>Magic Formula Portfolio</b>\nInvest: 10K per ticker\n\n'
-            for stock in data.to_dict(orient = 'record'):
-                message += '<b>' + stock['Name'] + '</b>\n'
-                message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
-                message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
-            return message
+            # Strategy
+            amount_per_stock = 10000
+            data['Quantity'] = round(amount_per_stock/data['CMP'],0)
+            
+            if formatMessage:
+                message = '<b>Magic Formula Portfolio</b>\nInvest: 10K per ticker\n\n'
+                for stock in data.to_dict(orient = 'record'):
+                    message += '<b>' + stock['Name'] + '</b>\n'
+                    message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
+                    message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
+                return message
+        except:
+            data = pd.DataFrame()
+            if formatMessage:
+                message = '<b>Magic Formula Portfolio</b>\nInvest: 10K per ticker\\n\n'
+                message += 'No ticker found'
+                return message
         return data
     
 class IncreasedHoldings:
     
     def screener(self, formatMessage = True):
-        url = 'https://www.screener.in/screens/452854/ChangeInHoldings/'
-        obj = ScreenerData(url)
-        data = obj.get()
-        data.reset_index( inplace = True, drop = True)
+        try:
+            url = 'https://www.screener.in/screens/452854/ChangeInHoldings/'
+            obj = ScreenerData(url)
+            data = obj.get()
+            data.reset_index( inplace = True, drop = True)
 
-        # Strategy
-        amount_per_stock = 10000
-        data['Quantity'] = round(amount_per_stock/data['CMP'],0)
-        
-        if formatMessage:
-            message = '<b>Increase DII & FII holdings</b>\nInvest: 10K per ticker\n\n'
-            for stock in data.to_dict(orient = 'record'):
-                message += '<b>' + stock['Name'] + '</b>\n'
-                message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
-                message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
-            return message
+            # Strategy
+            amount_per_stock = 10000
+            data['Quantity'] = round(amount_per_stock/data['CMP'],0)
+            
+            if formatMessage:
+                message = '<b>Increase DII & FII holdings</b>\nInvest: 10K per ticker\n\n'
+                for stock in data.to_dict(orient = 'record'):
+                    message += '<b>' + stock['Name'] + '</b>\n'
+                    message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
+                    message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
+                return message
+        except:
+            data = pd.DataFrame()
+            if formatMessage:
+                message = '<b>Increase DII & FII holdings</b>\nInvest: 10K per ticker\n\n'
+                message += 'No ticker found'
+                return message
         return data
     
 class GrowthPortfolio:
     
     def screener(self, formatMessage = True):
-        url = 'https://www.screener.in/screens/452860/GrowthCompanies/'
-        obj = ScreenerData(url)
-        data = obj.get()
-        data.reset_index( inplace = True, drop = True)
+        try:
+            url = 'https://www.screener.in/screens/452860/GrowthCompanies/'
+            obj = ScreenerData(url)
+            data = obj.get()
+            data.reset_index( inplace = True, drop = True)
 
-        # Strategy
-        amount_per_stock = 10000
-        data['Quantity'] = round(amount_per_stock/data['CMP'],0)
-        
-        if formatMessage:
-            message = '<b>Portfolio of Growers</b>\nInvest: 10K per ticker\n\n'
-            for stock in data.to_dict(orient = 'record'):
-                message += '<b>' + stock['Name'] + '</b>\n'
-                message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
-                message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
-            return message
+            # Strategy
+            amount_per_stock = 10000
+            data['Quantity'] = round(amount_per_stock/data['CMP'],0)
+            
+            if formatMessage:
+                message = '<b>Portfolio of Growers</b>\nInvest: 10K per ticker\n\n'
+                for stock in data.to_dict(orient = 'record'):
+                    message += '<b>' + stock['Name'] + '</b>\n'
+                    message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
+                    message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
+                return message
+        except:
+            data = pd.DataFrame()
+            if formatMessage:
+                message = '<b>Portfolio of Growers</b>\nInvest: 10K per ticker\n\n'
+                message += 'No ticker found'
+                return message
         return data
     
 class GoodLargeCapsCloseTo200DMA:
     
     def screener(self, formatMessage = True):
-        url = 'https://www.screener.in/screens/457921/CheapLargeCaps/'
-        obj = ScreenerData(url)
-        data = obj.get()
-        data.reset_index( inplace = True, drop = True)
+        try:
+            url = 'https://www.screener.in/screens/457921/CheapLargeCaps/'
+            obj = ScreenerData(url)
+            data = obj.get()
+            data.reset_index( inplace = True, drop = True)
 
-        # Strategy
-        amount_per_stock = 10000
-        data['Quantity'] = round(amount_per_stock/data['CMP'],0)
-        
-        if formatMessage:
-            message = '<b>Large Caps close to 200DMA</b>\nInvest: 10K per ticker\n\n'
-            for stock in data.to_dict(orient = 'record'):
-                message += '<b>' + stock['Name'] + '</b>\n'
-                message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
-                message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
-            return message
+            # Strategy
+            amount_per_stock = 10000
+            data['Quantity'] = round(amount_per_stock/data['CMP'],0)
+
+            if formatMessage:
+                message = '<b>Large Caps close to 200DMA</b>\nInvest: 10K per ticker\n\n'
+                for stock in data.to_dict(orient = 'record'):
+                    message += '<b>' + stock['Name'] + '</b>\n'
+                    message += 'LTP: ' + str(round(stock['CMP'], 2)) + '\n'
+                    message += 'Quantity: ' + str(round(stock['Quantity'], 2)) + '\n\n'
+                return message
+        except:
+            data = pd.DataFrame()
+            if formatMessage:
+                message = '<b>Large Caps close to 200DMA</b>\nInvest: 10K per ticker\n\n'
+                message += 'No ticker found'
+                return message
         return data
\ No newline at end of file
